import Vue from "vue";
import Router from "vue-router";

Vue.use(Router);

const originalPush = Router.prototype.push;
Router.prototype.push = function (location) {
  originalPush.call(this, location).catch((err) => err);
};
const originalReplace = Router.prototype.replace;
Router.prototype.replace = function (location) {
  originalReplace.call(this, location).catch((err) => err);
};

/* AppLayout */
import AppLayout from "@/layout/app/index.vue";

/* MainDefaultLayout */
import MainDefaultLayout from "@/layout/main/default/index.vue";

/**
 * Note: sub-menu only appear when route children.length >= 1
 * Detail see: https://panjiachen.github.io/vue-element-admin-site/guide/essentials/router-and-nav
 *
 * hidden: true                   if set true, item will not show in the sidebar(default is false)
 * alwaysShow: true               if set true, will always show the root menu
 *                                if not set alwaysShow, when item has more than one children route,
 *                                it will becomes nested mode, otherwise not show the root menu
 * redirect: noRedirect           if set noRedirect will no redirect in the breadcrumb
 * name:'router-name'             the name is used by <keep-alive> (must set!!!)
 * meta : {
    roles: ['admin','editor']    control the page roles (you can set multiple roles)
    title: 'title'               the name show in sidebar and breadcrumb (recommend set)
    icon: 'svg-name'/'el-icon-x' the icon show in the sidebar
    breadcrumb: false            if set false, the item will hidden in breadcrumb(default is true)
    activeMenu: '/example/list'  if set path, the sidebar will highlight the path you set
  }
 */

/**
 * allRoutes
 * a base page that does not have permission requirements
 * all roles can be accessed
 */
export const allRoutes = [
  {
    path: "/404",
    component: () => import("@/views/404.vue"),
    hidden: true,
  },

  {
    path: "/",
    redirect: "/hdlb",
  },

  {
    path: "/hdlb",
    // name: 'hdlb',
    component: AppLayout,
    meta: { title: "活动管理", icon: "el-icon-document" },
    // redirect: '/hdlb',

    children: [
      {
        path: "hdlxgl",
        // name: 'hdlxgl',
        component: MainDefaultLayout,
        children: [
          {
            path: "",
            name: "hdlxgl",
            component: () => import("@/views/xtgl/hdlxgl/index.vue"),
            meta: { title: "活动类型管理", roles: ["JLD"] },
          },
        ],
      },
      {
        path: "pfmb",
        // name: 'pfmb',
        component: MainDefaultLayout,
        meta: {
          roles: ["JLD"]
        },
        children: [
          {
            path: "",
            name: "pfmb",
            component: () => import("@/views/xtgl/pfmb/index.vue"),
            meta: { title: "评分模板" },
          },
          // {
          //   path: 'add',
          //   name: 'pfmb_add',
          //   hidden: true,
          //   component: () => import('@/views/xtgl/pfmb/add/index.vue'),
          //   meta: { title: '新增评分模板' },
          //   props: true,
          // },
          {
            path: "edit/:hp_id",
            name: "pfmb_edit",
            hidden: true,
            component: () => import("@/views/xtgl/pfmb/edit/index.vue"),
            meta: { title: "管理评分模板" },
            props: true,
          },
          // {
          //   path: 'edit_xz/:hp_id',
          //   name: 'pfmb_edit_xz',
          //   hidden: true,
          //   component: () => import('@/views/xtgl/pfmb/edit/index_xz.vue'),
          //   meta: { title: '管理评分模板' },
          //   props: true,
          // },
        ],
      },
      {
        path: "",
        name: "hdlb",
        component: () => import("@/views/hdlb/index.vue"),
        meta: { title: "活动列表" },
      },
      {
        path: "hdfq",
        name: "hdfq",
        hidden: true,
        component: () => import("@/views/hdlb/hdfq/index.vue"),
        meta: { title: "活动发起", roles: ["JLD"] },
      },
      {
        path: "hdbj/:hhg_id",
        name: "hdbj",
        hidden: true,
        component: () => import("@/views/hdlb/hdfq/edit.vue"),
        meta: { title: "活动编辑", roles: ["JLD"] },
        props: true,
      },
      {
        path: "sqjl/:hhg_id",
        hidden: true,
        component: MainDefaultLayout,
        meta: { title: "申请记录" },
        children: [
          {
            path: "",
            name: "sqjl",
            component: () => import("@/views/hdlb/sqjl/index.vue"),
            meta: { title: "申请记录" },
            props: true,
          },
          {
            path: "zjpf/:id/:difMark",
            name: "zjpf",
            component: () => import("@/views/hdlb/sqjl/zjpf/index.vue"),
            meta: { title: "专家评分", roles: ["ZJ"] },
            props: true,
          },
        ],
      },

      {
        path: "hdjg/:hhg_id",
        hidden: true,
        // props: true,
        component: MainDefaultLayout,
        meta: { title: "活动结果" },

        children: [
          {
            path: "",
            name: "hdjg",
            component: () => import("@/views/hdlb/hdjg/index.vue"),
            meta: { title: "活动结果" },
            props: true,
          },
          {
            path: "add",
            name: "hdjg_add",
            component: () => import("@/views/hdlb/hdjg/add/index.vue"),
            meta: { title: "新增活动结果" },
            props: true,
          },
          {
            path: "edit/:hb_id",
            name: "hdjg_edit",
            component: () => import("@/views/hdlb/hdjg/edit/index.vue"),
            meta: { title: "修改活动结果" },
            props: true,
          },
          {
            path: "xgpf/:hb_id",
            name: "hdjg_xgpf",
            component: () => import("@/views/hdlb/hdjg/xgpf/index.vue"),
            meta: { title: "修改评分" },
            props: true,
          },
        ],
      },
    ],
  },

  {
    path: "/xtgl",
    component: AppLayout,
    redirect: "/xtgl/zzjg",
    name: "xtgl",
    meta: { title: "系统管理", icon: "el-icon-setting", roles: ["JLD"] },
    children: [
      {
        path: "zzjg",
        // name: 'zzjg',
        component: MainDefaultLayout,
        meta: { title: "组织架构" },
        children: [
          {
            path: "",
            name: "zzjg",
            component: () => import("@/views/xtgl/zzjg/index.vue"),
            meta: { title: "组织架构" },
          },
        ],
      },
      {
        path: "zjk",
        // name: 'zjk',
        component: MainDefaultLayout,
        meta: { title: "专家库" },
        children: [
          {
            path: "",
            name: "zjk",
            component: () => import("@/views/xtgl/zjk/index.vue"),
            meta: { title: "专家库" },
          },
        ],
      },
      {
        path: "rzgl",
        // name: 'rzgl',
        component: MainDefaultLayout,
        meta: { title: "日志管理" },
        children: [
          {
            path: "",
            name: "rzgl",
            component: () => import("@/views/xtgl/rzgl/index.vue"),
            meta: { title: "日志管理" },
          },
        ],
      },
      // {
      //   path: "zdb",
      //   // name: 'zdb',
      //   component: MainDefaultLayout,
      //   meta: { title: "字典表" },
      //   children: [
      //     {
      //       path: "",
      //       name: "zdb",
      //       component: () => import("@/views/xtgl/zdb/index.vue"),
      //       meta: { title: "字典表" },
      //     },
      //   ],
      // },
    ],
  },

  {
    path: "/mhgl",
    component: AppLayout,
    redirect: "/mhgl/tzgg",
    name: "mhgl",
    meta: {
      title: "门户管理",
      icon: "el-icon-document-copy",
      roles: ["JLD"]
    },
    children: [
      {
        path: "tzgg",
        // name: 'tzgg',
        component: MainDefaultLayout,
        meta: { title: "通知公告", roles: ["JLD"] },
        children: [
          {
            path: "",
            name: "tzgg",
            component: () => import("@/views/mhgl/tzgg/index.vue"),
            meta: { title: "通知公告" },
          },
        ],
      },
      {
        path: "hdgg",
        // name: 'hdgg',
        component: MainDefaultLayout,
        meta: { title: "活动公告", roles: ["JLD"] },
        children: [
          {
            path: "",
            name: "hdgg",
            component: () => import("@/views/mhgl/hdgg/index.vue"),
            meta: { title: "活动公告" },
          },
        ],
      },
      {
        path: "yxzp",
        // name: 'yxzp',
        component: MainDefaultLayout,
        meta: { title: "优秀作品集", roles: ["JLD"] },
        children: [
          {
            path: "",
            name: "yxzp",
            component: () => import("@/views/mhgl/yxzp/index.vue"),
            meta: { title: "优秀作品集" },
          },
        ],
      },
      {
        path: "dmbwh",
        // name: 'yxzp',
        component: MainDefaultLayout,
        meta: { title: "代码表维护", roles: ["JLD"] },
        children: [
          {
            path: "",
            name: "dmbwh",
            component: () => import("@/views/mhgl/dmbwh/index.vue"),
            meta: { title: "代码表维护" },
          },
        ],
      },
      {
        path: "wjdc",
        // name: 'wjdc',
        component: MainDefaultLayout,
        meta: { title: "问卷调查", roles: ["JLD"] },
        children: [
          {
            path: "",
            name: "wjdc",
            component: () => import("@/views/mhgl/wjdc/index.vue"),
            meta: { title: "问卷调查" },
          },
          {
            path: "add",
            name: "wjdc_add",
            hidden: true,
            component: () => import("@/views/mhgl/wjdc/add/index.vue"),
            meta: { title: "新增问卷调查" },
            props: true,
          },
          {
            path: "edit/:id",
            name: "wjdc_edit",
            hidden: true,
            component: () => import("@/views/mhgl/wjdc/edit/index.vue"),
            meta: { title: "修改问卷调查" },
            props: true,
          },
        ],
      },
      {
        path: "tplb",
        // name: 'tplb',
        component: MainDefaultLayout,
        meta: { title: "投票列表", roles: ["JLD"] },
        children: [
          {
            path: "",
            name: "tplb",
            component: () => import("@/views/mhgl/tplb/index.vue"),
            meta: { title: "投票列表" },
          },
          {
            path: "add",
            name: "tplb_add",
            hidden: true,
            component: () => import("@/views/mhgl/tplb/add/index.vue"),
            meta: { title: "新增投票列表" },
            props: true,
          },
          {
            path: "edit/:id",
            name: "tplb_edit",
            hidden: true,
            component: () => import("@/views/mhgl/tplb/edit/index.vue"),
            meta: { title: "修改投票列表" },
            props: true,
          },
        ],
      },
    ],
  },

  // 404 page must be placed at the end !!!
  { path: "*", redirect: "/404", hidden: true },
];

function hasPermission(route, role) {
  if (Array.isArray(route?.meta?.roles)) {
    return route.meta.roles.includes(role)
  } else {
    return true
  }
}

function handleFilterRoutes(routes, role) {
  const res = []
  routes.forEach((route) => {
    const tmp = { ...route }
    if (hasPermission(tmp, role)) {
      if (tmp.children) {
        tmp.children = handleFilterRoutes(tmp.children, role)
      }
      res.push(tmp)
    }
  })
  return res
}

import { user } from '@/stores/user'
const filterRoutes = user.rolePower === 'all' ? allRoutes : handleFilterRoutes(allRoutes, user.roleName)

// function hasPermission(route, role) {
//   if (Array.isArray(route?.meta?.roles)) {
//     return route.meta.roles.includes(role);
//   } else {
//     return true;
//   }
// }

// function handleFilterRoutes(routes, role) {
//   const res = [];
//   routes.forEach((route) => {
//     const tmp = { ...route };
//     if (hasPermission(tmp, role)) {
//       if (tmp.children) {
//         tmp.children = handleFilterRoutes(tmp.children, role);
//       }
//       res.push(tmp);
//     }
//   });
//   return res;
// }

// import { user } from "@/stores/user";
// var filterRoutes;
// const ifPower = user.power;
// var announceMark = ifPower.announce;
// if (announceMark == 1) {
//   //设定了权限;
//   var aa = filterNewRoutes(allRoutes, ifPower);

//   filterRoutes = handleFilterRoutes(aa, user.roleName)
//   console.log(filterRoutes, "9999")
// } else {
//   //没有设定权限
//   filterRoutes = handleFilterRoutes(allRoutes, user.roleName);
// }
// // var filterRoutes = handleFilterRoutes(allRoutes, user.roleName);

// function filterNewRoutes(allRoutes, ifPower) {
//   var temp_routesArr = [];
//   allRoutes.forEach((item) => {
//     const tmp = { ...item };
//     if (item.name == "mhgl") {
//       delete item.meta.roles;
//       if (item.children) {
//         item.children.forEach((itemSec) => {
//           //将活动公告，通知公告的roles去除
//           if (itemSec.path == "hdgg" || itemSec.path == "tzgg") {
//             delete itemSec.meta.roles;
//           }
//         });
//       }
//     }
//     temp_routesArr.push(tmp);
//   });
//   return temp_routesArr;
// }
const createRouter = () =>
  new Router({
    // mode: 'history', // require service support
    scrollBehavior: () => ({ y: 0 }),
    routes: filterRoutes,
  });

const router = createRouter();

// Detail see: https://github.com/vuejs/vue-router/issues/1234#issuecomment-357941465
export function resetRouter() {
  const newRouter = createRouter();
  router.matcher = newRouter.matcher; // reset router
}

export default router;

/* 
=====================================================================================
=====================================================================================
*/

import getPageTitle from "@/utils/get-page-title";
router.beforeEach(async (to, from, next) => {
  document.title = getPageTitle(to.meta.title);
  console.groupCollapsed(
    `router ${from.meta.title || "新进入"} => ${to.meta.title}`
  );
  console.log("from :>> ", from);
  console.log("to :>> ", to);
  console.groupEnd();
  next();
});
